#include "TimerThree.h"
#include <Encoder.h>

//Mechanical Sytem Properties
  //full pendulum rotation =8000 Encoder counts
#define rot_Encoder_A 18    // rotary Encoderpin A an Pin 18
#define rot_Encoder_B 19    // rotary Encoderpin B an Pin 19
Encoder rotEnc(rot_Encoder_A, rot_Encoder_B);
  
/*
Connection to the IBT-2 board:
IBT-2 pin 1 (RPWM) to Arduino pin 5(PWM)
IBT-2 pin 2 (LPWM) to Arduino pin 6(PWM)
IBT-2 pins 3 (R_EN), 4 (L_EN), 7 (VCC) to Arduino 5V pin
IBT-2 pin 8 (GND) to Arduino GND
IBT-2 pins 5 (R_IS) and 6 (L_IS) not connected
*/
   int RPWM_Output = 6; // Arduino PWM output pin 6; connect to IBT-2 pin 1 (RPWM)
   int LPWM_Output = 7; // Arduino PWM output pin 7; connect to IBT-2 pin 2 (LPWM)
 
      
    const long    abtastung = 20000;          // Abtastzeitfenster in Mycrosekunden 
    const double  h = float(abtastung)/1000000.0;
    const double  cb = 2.0 * 3.141592/4000.0; // Schrittweite vom rotEncoder im Bogenmass, 2 Pi = 8000 Impulse je Umdrehung;
    const long    duration = 20 * 1000;  // Zeitdauer je Zustand 
       
    
    volatile long  zmax = 0; // Messung maximaler Pendelausschlag
    volatile long  z0 = 0;   //  Zaehlwerte der letzten drei Encoder Abtastungen (Pendelwinkel)
    volatile long  z1 = 0;   
    volatile long  z2 = 0;   
    volatile double rot_speed = 0.0;   //  Winkelgeschwindigkeit
    long  nullpunkt = 4000;
    long  angle = 0;
    long  StartTime = 0;
    int   state = 1;
 
void setup() {
  //PWM Frequenz auf 31 kHz setzen
  int myEraser = 7;
  TCCR4B &= ~myEraser;  // Timerregister fuer timer 4 loeschen
  int myPrescaler = 1;  // Vorteiler definieren       
  TCCR4B |= myPrescaler;  // Vorteiler laden
  Timer3.initialize(abtastung);         // initialize timer3, and set a period  
  Timer3.attachInterrupt(control);      // control Routine tastet den Encoder alle 20 ms  
  
  pinMode(RPWM_Output, OUTPUT);
  analogWrite(RPWM_Output, 0);
  pinMode(LPWM_Output, OUTPUT);
  analogWrite(LPWM_Output, 0);
  Serial.begin(115200); 
  
  //Initialize Encoder counts
  rotEnc.write(nullpunkt); //set when pendulum is hanging straight down due to gravity
  StartTime = millis();   
  randomSeed(analogRead(A0)); // Zufallszahlengenrator generieren
}

void control(){       // control Routine tastet den Encoder alle 20 ms 
    z2=z1; z1=z0;
    z0 = rotEnc.read();
    rot_speed = cb * (z0-z2)/(2 * h);
}

void Emagnet(int val){
   if(val >= 0){
    analogWrite(LPWM_Output, val);  //Pendel anziehen
    analogWrite(RPWM_Output, 0);
    }
    else {
    analogWrite(LPWM_Output, 0);   //Pendel abstoßen
    analogWrite(RPWM_Output, abs(val));
   }
}

void beschleunigen(){
const int deltaw = -50;   // mit delatw kann der Pendelauschlag optimiert werden
  
  if(rot_speed == 0.0){
    Emagnet(-255);       // bei Stillstand Pendel abstoßen
    delay(100);
    }
  else if(rot_speed < 0.0){ // omega ist negativ
    if( angle < (nullpunkt - deltaw)){
      Emagnet(-255); // nach dem Nullpunkt Pendel abstoßen
    }
    else if(angle > (nullpunkt + deltaw)) {
      Emagnet(255); //vor dem Nullpunkt Pendel anziehen
    }
    else Emagnet(0); // außerhalb des Wirkungsbereiches Magnet abschalten
  }
  else{ //omega positiv
    if(angle > (nullpunkt + deltaw)){
      Emagnet(-255); //Pendel abstoßen
    }
    else if(angle < (nullpunkt - deltaw)){
       Emagnet(255); //Pendel anziehen
    }
    else Emagnet(0); // Magnet abschalten
  }
}

void bremsen(){
    long diff = abs(nullpunkt - angle);
    if (diff < 40){
       Emagnet(255); // um den Nullpunkt herum anziehen
       delay(100);
    }
    else
    {
      Emagnet(0); // außerhalb vom Nullpunkt abschalten
    }
 }

void chaos(){
  long diff = abs(nullpunkt - angle);
  if (rot_speed == 0.0){
    Emagnet(-255); // bei Stillstand abstoßen
    delay(100);
  }
  else  if (diff < 400){
      Emagnet(-255 + random(511) );
      delay(20 + random(150)); // um den Nullpunkt herum anziehen oder abstoßen
    }
  else
    {
      Emagnet(0); // außerhalb vom Nullpunkt abschalten
    }
}

void rechts_wippen(){
  long diff = abs(nullpunkt - angle);
  if ( rot_speed == 0.0 && diff < 40 ){
     Emagnet(-255); // bei Stillstand abstoßen
     delay(100);
  }
  else if( rot_speed == 0.0 && diff >= 40){
     Emagnet(40); // bei Stillstand auf der rechten Seite kurz auf Anziehen schalten  
   }
  else if(angle < nullpunkt ){
     Emagnet(-255);  // maximal abstoßen
  }
}

void links_wippen(){
  long diff = abs(nullpunkt - angle);
  if ( rot_speed == 0.0 && diff < 40 ){
     Emagnet(-255); // bei Stillstand abstoßen
     delay(100);
  }
  else if( rot_speed == 0.0 && diff >= 40){
     Emagnet(40); // bei Stillstand auf der linken Seite kurz auf Anziehen schalten  
   }
  else if(angle > nullpunkt ){
     Emagnet(-255);  // maximal abstoßen
  }
}

void loop() {
   angle = rotEnc.read();
   switch(state) 
    { case 1:  beschleunigen();
               if (millis() - StartTime > duration){
                 state = 2;
                 StartTime = millis();
               };
               break;                
      case 2:  bremsen();
               if ( millis() - StartTime > duration){
                   state = 3; 
                   StartTime = millis();
                };
                break;  
      case 3:  chaos();
               if ( millis() - StartTime > duration){
                   state = 4; 
                   StartTime = millis();
                };
                break;      
      case 4:  rechts_wippen();
               if ( millis() - StartTime > duration){
                   state = 5; 
                   StartTime = millis();
                   
                 };
                break;   
      case 5:  links_wippen();
               if ( millis() - StartTime > duration){
                   state = 1; 
                   StartTime = millis();
                };
                break;   
     default : StartTime = millis();
               state = 1;
               break;             
    }
 } //End Loop
 
